/*
 *  linux/drivers/char/fbmem.c
 *
 *  Copyright (C) 1994 Martin Schaller
 *
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file COPYING in the main directory of this archive
 * for more details.
 */

/* typical usage: (lnx_init.c)
        X starts activity by:
        -open /dev/tty0, do VT_OPENQRY to get a free console
                -close it
        -use environment FRAMEBUFFER or "/dev/fb0current"
        -open it FOR RDONLY!
        -do a FBIOGET_VSCREENINFO
        -make a /dev/tty%d number that reflects the virtual console
        -open it as consoleFd
        -grab /dev/tty0
        -send VT_GETSTATE to consoleFd; remember active vt
        -open /dev/tty, send ioctl TIOCNOTTY, close
        -to consoleFD, send VT_ACTIVATE, VT_WAITACTIVATE,
                VT_GETMODE, VT_SETMODE (VT_PROCESS),
                KDSETMODE (KD_GRAPHICS)
        -to the fb, do a FBIOPUT_VSCREENINFO(0,0); close fb
 
        the fb gets opened later in fbdev.c::xf86MapVidMem for RDWR
                we can use the open-for-write to switch to graphics mode
 
        X shuts down by:
        -consoleFd, KDSETMODE(KD_TEXT), VT_GETMODE, VT_SETMODE(VT_AUTO),
        -send VT_ACTIVATE to restore previous console
        -close consoleFd
*/

#include <linux/types.h>
#include <linux/errno.h>
#include <linux/sched.h>
#include <linux/kernel.h>
#include <linux/major.h>
#include <linux/malloc.h>
#include <linux/mman.h>
#include <linux/tty.h>
#include <linux/kd.h>

#include <asm/segment.h>
#include <asm/page.h>
#include <asm/pgtable.h>
#include <asm/hardware.h>

#include <linux/fb.h>
#include "igs.h"

#define FB_MAJOR	29

#define FB_MODES_SHIFT    5	/* 32 modes per framebuffer */
#define FB_NUM_MINORS 	256	/* 256 Minors		    */
#define FB_MAX		(FB_NUM_MINORS / (1 << FB_MODES_SHIFT))

#define GET_INODE(i) MKDEV(FB_MAJOR, (i) << FB_MODES_SHIFT)
#define GET_FB_IDX(node) (MINOR(node) >> FB_MODES_SHIFT)
#define GET_FB_VAR_IDX(node) (MINOR(node) & ((1 << FB_MODES_SHIFT)-1)) 

struct fb_ops *registered_fb[FB_MAX];
struct fb_var_screeninfo *registered_fb_var[FB_MAX];
int registered_fb_var_num[FB_MAX];
int fb_curr_open[FB_MAX];
int fb_open_count[FB_MAX];

extern void do_blank_screen(int nopowersave);
extern void do_unblank_screen(void);

static inline int PROC_CONSOLE(void)
{
	if (!current->tty)
		return fg_console;

	if (current->tty->driver.type != TTY_DRIVER_TYPE_CONSOLE)
		/* XXX Should report error here? */
		return fg_console;

	if (MINOR(current->tty->device) < 1)
		return fg_console;

	return MINOR(current->tty->device) - 1;
}

static int 
fb_read(struct inode *inode, struct file *file, char *buf, int count)
{
	unsigned long p = file->f_pos;
	struct fb_ops *fb = registered_fb[GET_FB_IDX(inode->i_rdev)];
	struct fb_fix_screeninfo fix;
	char *base_addr;
	int copy_size;

	if (! fb)
		return -ENODEV;
	if (count < 0)
		return -EINVAL;

	fb->fb_get_fix(&fix,PROC_CONSOLE());
	base_addr=(char *) fix.smem_start;
	copy_size=(count + p <= fix.smem_len ? count : fix.smem_len - p);
	memcpy_tofs(buf, base_addr+p, copy_size);
	file->f_pos += copy_size;
	return copy_size;
}

static int 
fb_write(struct inode *inode, struct file *file, const char *buf, int count)
{
	unsigned long p = file->f_pos;
	struct fb_ops *fb = registered_fb[GET_FB_IDX(inode->i_rdev)];
	struct fb_fix_screeninfo fix;
	char *base_addr;
	int copy_size;

	if (! fb)
		return -ENODEV;
	if (count < 0)
		return -EINVAL;
	fb->fb_get_fix(&fix, PROC_CONSOLE());
	base_addr=(char *) fix.smem_start;
	copy_size=(count + p <= fix.smem_len ? count : fix.smem_len - p);
	memcpy_fromfs(base_addr+p, buf, copy_size); 
	file->f_pos += copy_size;
	return copy_size;
}


static int 
fb_ioctl(struct inode *inode, struct file *file, unsigned int cmd,
	 unsigned long arg)
{
	struct fb_ops *fb = registered_fb[GET_FB_IDX(inode->i_rdev)];
	struct fb_cmap cmap;
	struct fb_var_screeninfo var;
	struct fb_fix_screeninfo fix;

	int i,fbidx,vidx;
	
	if (! fb)
		return -ENODEV;
	switch (cmd) {
	case FBIOGET_VSCREENINFO:
		i = verify_area(VERIFY_WRITE, (void *) arg, 
				sizeof(struct fb_var_screeninfo));
		if (i) return i;
		fbidx=GET_FB_IDX(inode->i_rdev);
		vidx=GET_FB_VAR_IDX(inode->i_rdev);
		if (! vidx) /* ask device driver for current */
			i=fb->fb_get_var(&var, PROC_CONSOLE());
		else
			var=registered_fb_var[fbidx][vidx-1];
		memcpy_tofs((void *) arg, &var, sizeof(var));
		return i;
	case FBIOPUT_VSCREENINFO:
		i = verify_area(VERIFY_WRITE, (void *) arg, 
				sizeof(struct fb_var_screeninfo));
		if (i) return i;
		memcpy_fromfs(&var, (void *) arg, sizeof(var));
		i=fb->fb_set_var(&var, PROC_CONSOLE());
		memcpy_tofs((void *) arg, &var, sizeof(var));
		fbidx=GET_FB_IDX(inode->i_rdev);
		vidx=GET_FB_VAR_IDX(inode->i_rdev);
		if (! i && vidx)
			registered_fb_var[fbidx][vidx-1]=var;
		return i;
	case FBIOGET_FSCREENINFO:
		i = verify_area(VERIFY_WRITE, (void *) arg, 
				sizeof(struct fb_fix_screeninfo));
		if (i)	return i;
		i=fb->fb_get_fix(&fix, PROC_CONSOLE());
		memcpy_tofs((void *) arg, &fix, sizeof(fix));
		return i;
	case FBIOPUTCMAP:
		i = verify_area(VERIFY_READ, (void *) arg,
				sizeof(struct fb_cmap));
		if (i) return i;
		memcpy_fromfs(&cmap, (void *) arg, sizeof(cmap));
		i = verify_area(VERIFY_READ, (void *) cmap.red, 
				cmap.len * sizeof(unsigned short));
		if (i) return i;
		i = verify_area(VERIFY_READ, (void *) cmap.green, 
				cmap.len * sizeof(unsigned short));
		if (i) return i;
		i = verify_area(VERIFY_READ, (void *) cmap.blue, 
				cmap.len * sizeof(unsigned short));
		if (i) return i;
		if (cmap.transp) {
			i = verify_area(VERIFY_READ, (void *) cmap.transp, 
					cmap.len * sizeof(unsigned short));
			if (i) return i;
		}
		return (fb->fb_set_cmap(&cmap, 0, PROC_CONSOLE()));
	case FBIOGETCMAP:
		i = verify_area(VERIFY_READ, (void *) arg,
				sizeof(struct fb_cmap));
		if (i)	return i;
		memcpy_fromfs(&cmap, (void *) arg, sizeof(cmap));
		i = verify_area(VERIFY_WRITE, (void *) cmap.red, 
				cmap.len * sizeof(unsigned short));
		if (i) return i;
		i = verify_area(VERIFY_WRITE, (void *) cmap.green, 
				cmap.len * sizeof(unsigned short));
		if (i) return i;
		i = verify_area(VERIFY_WRITE, (void *) cmap.blue, 
				cmap.len * sizeof(unsigned short));
		if (i) return i;
		if (cmap.transp) {
			i = verify_area(VERIFY_WRITE, (void *) cmap.transp, 
					cmap.len * sizeof(unsigned short));
			if (i) return i;
		}
		return (fb->fb_get_cmap(&cmap, 0, PROC_CONSOLE()));
	case FBIOPAN_DISPLAY:
		i = verify_area(VERIFY_WRITE, (void *) arg, 
				sizeof(struct fb_var_screeninfo));
		if (i) return i;
		memcpy_fromfs(&var, (void *) arg, sizeof(var));
		i=fb->fb_pan_display(&var, PROC_CONSOLE());
		memcpy_tofs((void *) arg, &var, sizeof(var));
		fbidx=GET_FB_IDX(inode->i_rdev);
		vidx=GET_FB_VAR_IDX(inode->i_rdev);
		if (! i && vidx)
			registered_fb_var[fbidx][vidx-1]=var;
		return i;
	default:
		return (fb->fb_ioctl(inode, file, cmd, arg, PROC_CONSOLE()));
	}
}

static int 
fb_mmap(struct inode *inode, struct file *file, struct vm_area_struct * vma)
{
	struct fb_ops *fb = registered_fb[GET_FB_IDX(inode->i_rdev)];
	struct fb_fix_screeninfo fix;

	if (! fb)
		return -ENODEV;
	fb->fb_get_fix(&fix, PROC_CONSOLE());
	if ((vma->vm_end - vma->vm_start + vma->vm_offset) > fix.smem_len)
		return -EINVAL;
	vma->vm_offset += fix.smem_start;
	if (vma->vm_offset & ~PAGE_MASK)
		return -ENXIO;
#if 0
	if (m68k_is040or060) {
		pgprot_val(vma->vm_page_prot) &= _CACHEMASK040;
		/* Use write-through cache mode */
		pgprot_val(vma->vm_page_prot) |= _PAGE_CACHE040W;
	}
#endif

#ifdef CONFIG_ARCH_VNC
        // compenstate for the action of remap_page_range, and offset back up to 0x88000000
	// printk(KERN_DEBUG "remap to %x\n", vma->vm_offset+PAGE_OFFSET+VIDEO_RAM_BASE-IO_BASE_VGA);
        if (remap_page_range(vma->vm_start, vma->vm_offset+PAGE_OFFSET+VIDEO_RAM_BASE-IO_BASE_VGA,
                             vma->vm_end - vma->vm_start, __pgprot(pgprot_val(vma->vm_page_prot) & ~PTE_CACHEABLE)))
                return -EAGAIN;
#else
	if (remap_page_range(vma->vm_start, vma->vm_offset,
			     vma->vm_end - vma->vm_start, vma->vm_page_prot))
		return -EAGAIN;
#endif
	vma->vm_inode = inode;
	inode->i_count++;
	return 0;
}

/*
 * open
 *
 * device 0 means "current mode"
 * device 1-31 specifies a preset graphics mode
 * after the open, a FBIOPUT_VSCREENINFO can change
 * the aspect of one mode to another
 */
static int
fb_open(struct inode *inode, struct file *file)
{
	int fbidx=GET_FB_IDX(inode->i_rdev);
	int vidx=GET_FB_VAR_IDX(inode->i_rdev);
	struct fb_ops *fb = registered_fb[fbidx];
	int err;
	
if (vidx > registered_fb_var_num[fbidx]) printk("fb_open: vidx too high %x, %x\n",vidx,registered_fb_var_num[fbidx]);
if (fb_curr_open[fbidx] && fb_curr_open[fbidx] != vidx) printk("fb_open: already open %x\n",fb_curr_open[fbidx]);

	if (vidx > registered_fb_var_num[fbidx])
		return -EINVAL;
	if (fb_curr_open[fbidx] && fb_curr_open[fbidx] != vidx)
		return -EBUSY;

        /* only set parameters if opened writeable */
        if (file->f_mode & 2) {
	// printk(KERN_DEBUG "fb_open for write into %x vidx width varinfo based from %x\n",vidx,registered_fb_var[fbidx]);
                if (vidx==0) vidx = 1;  /* open the "current" device is the same as opening the "default" */
		if ((err=fb->fb_set_var(registered_fb_var[fbidx] + vidx-1, PROC_CONSOLE())))
			return err;
        }
 
//      if (! vidx)             /* fb?current for read always succeeds */
//              return 0;
 
	fb_curr_open[fbidx] = vidx;
	fb_open_count[fbidx]++;
	return 0;
}

static void 
fb_release(struct inode *inode, struct file *file)
{
	int fbidx=GET_FB_IDX(inode->i_rdev);
//	int vidx=GET_FB_VAR_IDX(inode->i_rdev);
//      if (! vidx)
//              return;
        if (!(--fb_open_count[fbidx])) {
 printk("fb_release going back to KD_TEXT\n");
                fb_curr_open[fbidx]=0;
                igs_setmode(0);
                do_unblank_screen();
        }
}

static struct file_operations fb_fops = {
	NULL,		/* lseek	*/
	fb_read,	/* read		*/
	fb_write,	/* write	*/
	NULL,		/* readdir 	*/
	NULL,		/* select 	*/
	fb_ioctl,	/* ioctl 	*/
	fb_mmap,	/* mmap		*/
	fb_open,	/* open 	*/
	fb_release,	/* release 	*/
	NULL		/* fsync 	*/
};

int
register_framebuffer(char *id, int *node, struct fb_ops *fbops, int fbvar_num, 
		     struct fb_var_screeninfo *fbvar)
{
	int i;
	for (i = 0 ; i < FB_MAX; i++)
		if (! registered_fb[i])
			break;
	if (i == FB_MAX)
		return -ENXIO;
	registered_fb[i]=fbops;
	registered_fb_var[i]=fbvar;
	registered_fb_var_num[i]=fbvar_num;
	*node=GET_INODE(i);
	return 0;
}

int
unregister_framebuffer(int node)
{
	int i=GET_FB_IDX(node);
	if (! registered_fb[i])
		return -EINVAL; 
	registered_fb[i]=NULL;
	registered_fb_var[i]=NULL;
	return 0;
}

#ifdef CONFIG_ARCH_VNC
// 0 == /dev/fb0current
// 1 == /dev/fb0default comes up in color640x480x24
// 2 == /dev/fb0mono640x480x1
// 3 == /dev/fb0color1024x768x16 
// 4 == /dev/fb0color800x600x16 
// 5 == /dev/fb0color640x480x16 
//new modes come in here...
// 6 == IGS_640x480x8
// 7 == IGS_800x600x8
// 8 == IGS_1024x768x8
// 9 == IGS_800x600x24
//10 == IGS_1280x1024x8
//11 == IGS_1600x1200x8
//
// 22 == /dev/fb0ntsc
// 23 == /dev/fb0pal
const struct fb_fix_screeninfo vnc_fix = {
	"Corel VNC",
	IO_BASE_VGA, SZ_2M,
	FB_TYPE_PACKED_PIXELS, 0,
	FB_VISUAL_TRUECOLOR,
	0,0,0,
	0,		// variable line length
	{0,0,0,0,0,0,0,0,0}
};


// You can reference this array with the IGS_* constants MINUS ONE!

struct fb_var_screeninfo vnc_grfmodes[]	= {
// 640 x 480 x 24
{
	640, 480,
	640, 480,
	0, 0,
	24,
	0,
	{0,8,0},{8,8,0},{16,8,0},{0,0,0},
	0,
	FB_ACTIVATE_NOW,
	30, 40,
	FB_ACCEL_NONE,	// !! fix this later
	0,0,0,0,0,0,0,0,
	0,{0,0,0,0,0,0}
},

// 640 x 480 x 1
{
	640, 480,
	640, 480,
	0, 0,
	1,
	1,
	{0,0,0},{0,0,0},{0,0,0},{0,0,0},
	0,
	FB_ACTIVATE_NOW,
	30, 40,
	FB_ACCEL_NONE,	// !! fix this later
	0,0,0,0,0,0,0,0,
	0,{0,0,0,0,0,0}
},
// 1024 x 768 x 16
{
	1024, 768,
	1024, 768,
	0, 0,
	16,
	0,
	{0,5,0},{5,6,0},{11,5,0},{0,0,0},
	0,
	FB_ACTIVATE_NOW,
	30, 40,
	FB_ACCEL_NONE,	// !! fix this later
	0,0,0,0,0,0,0,0,
	0,{0,0,0,0,0,0}
},

// 800 x 600 x 16
{
	800, 600,
	800, 600,
	0, 0,
	16,
	0,
	{0,5,0},{5,6,0},{11,5,0},{0,0,0},
	0,
	FB_ACTIVATE_NOW,
	30, 40,
	FB_ACCEL_NONE,	// !! fix this later
	0,0,0,0,0,0,0,0,
	0,{0,0,0,0,0,0}
},

// 640 x 480 x 16

{
	640, 480,
	640, 480,
	0, 0,
	16,
	0,
//	{0,5,0},{5,6,0},{11,5,0},{0,0,0},
	{11,5,0},{5,6,0},{0,5,0},{0,0,0},
	0,
	FB_ACTIVATE_NOW,
	30, 40,
	FB_ACCEL_NONE,	// !! fix this later
	0,0,0,0,0,0,0,0,
	0,{0,0,0,0,0,0}
},

// 640 x 480 x 8

{
	640, 480,
	640, 480,
	0, 0,
	8,
	0,
	{0,0,0},{0,0,0},{0,0,0},{0,0,0},
	0,
	FB_ACTIVATE_NOW,
	30, 40,
	FB_ACCEL_NONE,	// !! fix this later
	0,0,0,0,0,0,0,0,
	0,{0,0,0,0,0,0}
},

// 800 x 600 x 8

{
	800, 600,
	800, 600,
	0, 0,
	8,
	0,
	{0,0,0},{0,0,0},{0,0,0},{0,0,0},
	0,
	FB_ACTIVATE_NOW,
	30, 40,
	FB_ACCEL_NONE,	// !! fix this later
	0,0,0,0,0,0,0,0,
	0,{0,0,0,0,0,0}
},


// 1024 x 768 x 8

{
	1024,768,
	1024,768,
	0, 0,
	8,
	0,
	{0,0,0},{0,0,0},{0,0,0},{0,0,0},
	0,
	FB_ACTIVATE_NOW,
	30, 40,
	FB_ACCEL_NONE,	// !! fix this later
	0,0,0,0,0,0,0,0,
	0,{0,0,0,0,0,0}
},


// 800 x 600 x 24
{
	800, 600,
	800, 600,
	0, 0,
	24,
	0,
	{0,8,0},{8,8,0},{16,8,0},{0,0,0},
	0,
	FB_ACTIVATE_NOW,
	30, 40,
	FB_ACCEL_NONE,	// !! fix this later
	0,0,0,0,0,0,0,0,
	0,{0,0,0,0,0,0}
},


// 1280 x 1024 x 8

{
	1280,1024,
	1280,1024,
	0, 0,
	8,
	0,
	{0,0,0},{0,0,0},{0,0,0},{0,0,0},
	0,
	FB_ACTIVATE_NOW,
	30, 40,
	FB_ACCEL_NONE,	// !! fix this later
	0,0,0,0,0,0,0,0,
	0,{0,0,0,0,0,0}
},


// 1600 x 1200 x 8

{
	1600,1200,
	1600,1200,
	0, 0,
	8,
	0,
	{0,0,0},{0,0,0},{0,0,0},{0,0,0},
	0,
	FB_ACTIVATE_NOW,
	30, 40,
	FB_ACCEL_NONE,	// !! fix this later
	0,0,0,0,0,0,0,0,
	0,{0,0,0,0,0,0}
}


};



int vnc_get_fix(struct fb_fix_screeninfo *pfix, int i) {
	memcpy(pfix, &vnc_fix, sizeof (struct fb_fix_screeninfo));
	return 0;
}
int vnc_get_var(struct fb_var_screeninfo *pvar, int i) {
	memcpy(pvar, vnc_grfmodes + fb_curr_open[0], sizeof (struct fb_var_screeninfo));
	return 0;	
}
int vnc_set_var(struct fb_var_screeninfo *pvar, int i) {
//	int mode;

//printk("vnc_set_var: with xres: %d bbp: %d\n",pvar->xres,pvar->bits_per_pixel);
//printk("vnc_set_var: pvar_info:%x\n",pvar);

	do_blank_screen(1);

	// at this point, we should check the passed in parameters,
	// keep a local copy, and set the graphics mode
	// but this lousy code will have to do for a while

	if (pvar->xres==640)
	{
		if (pvar->bits_per_pixel==24)
			igs_setmode(IGS_640x480x24);
		else if (pvar->bits_per_pixel==16)
			igs_setmode(IGS_640x480x16);
		else if (pvar->bits_per_pixel==8)
			igs_setmode(IGS_640x480x8);
		else if (pvar->bits_per_pixel==1)
			igs_setmode(IGS_640x480x1);
		else return -EINVAL;
	}
	else if (pvar->xres==800)
	{
		if (pvar->bits_per_pixel==24)
			igs_setmode(IGS_800x600x24);
		else if (pvar->bits_per_pixel==16)
			igs_setmode(IGS_800x600x16);
		else if (pvar->bits_per_pixel==8)
			igs_setmode(IGS_800x600x8);
		else return -EINVAL;
	}
	else if (pvar->xres==1024)
	{
//		if (pvar->bits_per_pixel==24)
//			igs_setmode(IGS_1024x768x600x24);
//		else
		if (pvar->bits_per_pixel==16)
			igs_setmode(IGS_1024x768x16);
		else if (pvar->bits_per_pixel==8)
			igs_setmode(IGS_1024x768x8);
		else return -EINVAL;
	}
	else if (pvar->xres==1280 && pvar->bits_per_pixel==8) 
		igs_setmode(IGS_1280x1024x8);
//	else if (pvar->xres==1600 && pvar->bits_per_pixel==8) 
//		igs_setmode(IGS_1600x1024x8);
	else 
		return -EINVAL;
	// success
	return 0;
}
int vnc_get_cm(struct fb_cmap* pcm, int a, int b) {
	return -EINVAL;
}
int vnc_set_cm(struct fb_cmap* pcm, int a, int b) {
	return -EINVAL;
}
int vnc_pan(struct fb_var_screeninfo *pvar, int i) {
	return -EINVAL;
}
int vnc_ioctl(struct inode * pi, struct file * pf, unsigned int a,
	unsigned long b, int c)
{
	return -EINVAL;
}



struct fb_ops vnc_fb_ops = {
	vnc_get_fix,
	vnc_get_var, vnc_set_var,
	vnc_get_cm, vnc_set_cm,
	vnc_pan,
	vnc_ioctl
};
#endif

void
fbmem_init(void)
{
	if (register_chrdev(FB_MAJOR,"fb",&fb_fops))
		printk("unable to get major %d for fb devs\n", FB_MAJOR);
#ifdef CONFIG_ARCH_VNC
        {
        int i;
        // change the registrations numbers to #defines !!
        register_framebuffer("Corel VNC:fb0",&i,&vnc_fb_ops,
	11, //sizeof (vnc_grfmodes) / sizeof(struct(fb_var_screeninfo)),
	vnc_grfmodes);

//printk("frame buffer register returned inode of %x\n",i);
        }
#endif
}

